﻿using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Windows.Input;
using Microsoft.Maui.Controls;
using RobobinApp.Views;
using Plugin.BLE;
using Plugin.BLE.Abstractions.Contracts;
using Plugin.BLE.Abstractions.Exceptions;
using Plugin.BLE.Abstractions.EventArgs;
using System.Threading.Tasks;
using Plugin.BLE.Abstractions;
using RobobinApp.Models;
using System.Net.NetworkInformation; 
using System.Collections.Generic;

using Windows.Networking.Connectivity;
using Windows.Devices.WiFi;
using System.Reflection.PortableExecutable;
using Plugin.BLE.Windows;
using System.Text;

namespace RobobinApp.ViewModels
{
    public class ConnectionPageViewModel : BaseViewModel
    {
        private BluetoothDevice _selectedDevice;
        private IDevice _connectedDevice;
        public ICommand GoHomeCommand { get; }
        public ObservableCollection<BluetoothDevice> BluetoothDevices { get; }
        public ObservableCollection<WifiNetwork> WifiNetworks { get; }

        public ICommand ConnectCommand { get; }
        public ICommand SendWifiInfoCommand { get; }
        public ICommand DisconnectCommand { get; }
        private bool _isConnected;
        private string ssid;
        private string password;
        public bool IsBluetoothDeviceSelectionVisible => !IsWifiNetworkSelectionVisible;
        public const string SendReceiveServiceUUID = "00000001-710e-4a5b-8d75-3e5b444bc3cf";
        public const string rxUUID = "00000002-710e-4a5b-8d75-3e5b444bc3cf";
        public const string wxUUID = "00000003-710e-4a5b-8d75-3e5b444bc3cf";

        public bool IsConnected
        {
            get => _isConnected;
            set
            {
                _isConnected = value;
                OnPropertyChanged();
                OnPropertyChanged(nameof(ConnectButtonText));  // Update button text when connection state changes
            }
        }

        public string SSID
        {
            get => ssid;
            set
            {
                ssid = value;
                OnPropertyChanged(nameof(SSID));
            }
        }

        public string Password
        {
            get => password;
            set
            {
                password = value;
                OnPropertyChanged(nameof(Password));
            }
        }

        private readonly IAdapter _adapter;
        private readonly IBluetoothLE _bluetoothLE;
        public string ConnectButtonText => IsConnected ? "Disconnect" : "Connect";
        private bool _isWifiNetworkSelectionVisible;
        private WifiNetwork _selectedWifiNetwork;

        public bool IsWifiNetworkSelectionVisible
        {
            get => _isWifiNetworkSelectionVisible;
            set
            {
                _isWifiNetworkSelectionVisible = value;
                OnPropertyChanged();
                OnPropertyChanged(nameof(IsBluetoothDeviceSelectionVisible));
            }
        }

        public ConnectionPageViewModel()
        {
            BluetoothDevices = new ObservableCollection<BluetoothDevice>();
            WifiNetworks = new ObservableCollection<WifiNetwork>();
            ConnectCommand = new Command(OnConnect);
            DisconnectCommand = new Command(OnDisconnect);
            SendWifiInfoCommand = new Command(OnSendWifiInfo);
            GoHomeCommand = new Command(async () => await OnGoHome());
            ConnectCommand = new Command(OnToggleConnection);
            _bluetoothLE = CrossBluetoothLE.Current;
            _adapter = _bluetoothLE.Adapter;
            _adapter.DeviceDiscovered += OnDeviceDiscovered;
            IsWifiNetworkSelectionVisible = false;

            Debug.WriteLine("Checking and requesting Bluetooth permissions.");
            CheckAndRequestBluetoothPermissions();
            //ScanForWifiNetworks();
        }
        private async void ScanForWifiNetworks()
        {
            WifiNetworks.Clear();
            Debug.WriteLine("Scanning for Wi-Fi networks...");

            var wifiAdapter = await GetWifiAdapterAsync();
            if (wifiAdapter == null)
            {
                Debug.WriteLine("Wi-Fi adapter not found.");
                return;
            }

            await wifiAdapter.ScanAsync();
            Debug.Write("Scanning for networks");
            var networks = wifiAdapter.NetworkReport.AvailableNetworks;

            var addedSsids = new HashSet<string>(); // To keep track of added SSIDs

            foreach (var network in networks)
            {
                Debug.Write("This network exists: " + network.Ssid);

                if (addedSsids.Add(network.Ssid))
                {
                    var wifiNetwork = new WifiNetwork
                    {
                        SSID = network.Ssid,
                        SignalStrength = network.NetworkRssiInDecibelMilliwatts
                    };

                    Microsoft.Maui.Controls.Device.BeginInvokeOnMainThread(() => WifiNetworks.Add(wifiNetwork));
                    Debug.WriteLine($"Found Wi-Fi network: {wifiNetwork.SSID}, Signal Strength: {wifiNetwork.SignalStrength} dBm");
                }
            }
        }


        private async Task<WiFiAdapter> GetWifiAdapterAsync()
        {
            var result = await WiFiAdapter.FindAllAdaptersAsync();
            return result.FirstOrDefault(); // Get the first available Wi-Fi adapter
        }

        private async void OnSendWifiInfo(object obj)
        {
            if (SelectedWifiNetwork != null)
            {
                Debug.WriteLine("Wifi been selected");
                Debug.WriteLine($"Selected Network {SelectedWifiNetwork.SSID}");
            }
            if (!_isConnected || _connectedDevice == null)
            {
                Debug.WriteLine("Cannot send WiFi information: No device is connected.");
                await Application.Current.MainPage.DisplayAlert("Error", "No device is connected.", "OK");
                return;
            }

            try
            {
                Debug.WriteLine($"Sending WiFi information to {_connectedDevice.Name}...");


                var services = await _connectedDevice.GetServicesAsync();
                var uartService = services.FirstOrDefault(s => s.Id == Guid.Parse("6e400001-b5a3-f393-e0a9-e50e24dcca9e"));

                if (uartService == null)
                {
                    Debug.WriteLine("UART service not found.");
                    await Application.Current.MainPage.DisplayAlert("Error", "UART service not found on the device.", "OK");
                    return;
                }

                var characteristics = await uartService.GetCharacteristicsAsync();
                var rxCharacteristic = characteristics.FirstOrDefault(c => c.Id == Guid.Parse(rxUUID));

                if (rxCharacteristic == null)
                {
                    Debug.WriteLine("RX characteristic not found.");
                    await Application.Current.MainPage.DisplayAlert("Error", "RX characteristic not found on the device.", "OK");
                    return;
                }

                string wifiInfo = $"{SSID},{Password}";
                byte[] data = System.Text.Encoding.UTF8.GetBytes(wifiInfo);

                await rxCharacteristic.WriteAsync(data);

                Debug.WriteLine("WiFi information sent successfully.");
                await Application.Current.MainPage.DisplayAlert("Success", "WiFi information sent successfully.", "OK");
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Error sending WiFi information: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Error", $"Failed to send WiFi information: {ex.Message}", "OK");
            }
        }


        public BluetoothDevice SelectedDevice
        {
            get => _selectedDevice;
            set
            {
                _selectedDevice = value;
                OnPropertyChanged();
            }
        }
        public WifiNetwork SelectedWifiNetwork
        {
            get => _selectedWifiNetwork;
            set
            {
                _selectedWifiNetwork = value;
                OnPropertyChanged();
            }
        }
        private async void OnToggleConnection()
        {
            if (IsConnected)
            {
                OnDisconnect();
            }
            else
            {
                OnConnect();
            }
        }
        private async void CheckAndRequestBluetoothPermissions()
        {
            Debug.WriteLine("Checking Bluetooth permissions.");
            var status = await Permissions.CheckStatusAsync<Permissions.Bluetooth>();
            if (status != PermissionStatus.Granted)
            {
                Debug.WriteLine("Bluetooth permission not granted, requesting permission.");
                status = await Permissions.RequestAsync<Permissions.Bluetooth>();
            }

            if (status == PermissionStatus.Granted)
            {
                Debug.WriteLine("Bluetooth permission granted, proceeding to scan devices.");
                ScanDevices();
            }
            else
            {
                Debug.WriteLine("Bluetooth permission denied.");
                await Application.Current.MainPage.DisplayAlert("Permissions", "Bluetooth scan permission is required to discover devices.", "OK");
            }
        }

        public async Task OnGoHome()
        {
            Debug.WriteLine("Navigating to home page.");
            await Application.Current.MainPage.Navigation.PushAsync(new MainPage());
        }

        private async void ScanDevices()
        {
            BluetoothDevices.Clear();
            Debug.WriteLine("Cleared existing Bluetooth devices. Starting scan...");

            try
            {
                if (!_bluetoothLE.IsAvailable)
                {
                    Debug.WriteLine("Bluetooth is not available.");
                    await Application.Current.MainPage.DisplayAlert("Bluetooth", "Bluetooth is not available.", "OK");
                    return;
                }

                if (!_bluetoothLE.IsOn)
                {
                    Debug.WriteLine("Bluetooth is turned off.");
                    await Application.Current.MainPage.DisplayAlert("Bluetooth", "Please turn on Bluetooth.", "OK");
                    return;
                }

                Debug.WriteLine("Starting scanning for devices...");
                await _adapter.StartScanningForDevicesAsync();
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Error during scanning: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Error", $"Failed to scan for devices: {ex.Message}", "OK");
            }
        }

        private void OnDeviceDiscovered(object sender, DeviceEventArgs e)
        {
            Debug.WriteLine($"Discovered device: {e.Device.Name ?? e.Device.Id.ToString()}");

            var bluetoothDevice = new BluetoothDevice(
                string.IsNullOrWhiteSpace(e.Device.Name) ? e.Device.Id.ToString() : e.Device.Name,
                e.Device.Id.ToString()
            );

            if (!BluetoothDevices.Any(d => d.MacAddress == bluetoothDevice.MacAddress))
            {
                Microsoft.Maui.Controls.Device.BeginInvokeOnMainThread(() => BluetoothDevices.Add(bluetoothDevice));
            }
        }


        private async Task<IDevice> TryConnectAsync(Guid deviceId, int retryCount = 3)
        {
            IDevice device = null;
            for (int attempt = 1; attempt <= retryCount; attempt++)
            {
                try
                {
                    Debug.WriteLine($"Attempt {attempt}: Connecting to device {deviceId}");
                    var cancellationTokenSource = new CancellationTokenSource(); // No timeout specified
                    device = await _adapter.ConnectToKnownDeviceAsync(deviceId, new ConnectParameters(false, true), cancellationTokenSource.Token);
                    return device;
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"Attempt {attempt} failed: {ex.Message}");
                    if (attempt == retryCount)
                    {
                        throw;
                    }
                }
            }
            return device;
        }
        private async void OnConnect()
        {
            if (SelectedDevice == null)
            {
                Debug.WriteLine("No device selected for connection.");
                return;
            }

            Debug.WriteLine($"Attempting to connect to {SelectedDevice.Name}, MAC Address {SelectedDevice.MacAddress}");

            try
            {
                await _adapter.StopScanningForDevicesAsync();
                IsConnected = true;

                _connectedDevice = await TryConnectAsync(Guid.Parse(SelectedDevice.MacAddress));

                if (_connectedDevice != null)
                {
                    Debug.WriteLine($"Successfully connected to {SelectedDevice.Name}.");
                    var services = await _connectedDevice.GetServicesAsync();
                    IsWifiNetworkSelectionVisible = true;
                    // Iterate through the discovered services
                    foreach (var service in services)
                    {
                        Debug.WriteLine($"Service: ID={service.Id}, Name={service.Name}");

                        if (service.Id.Equals(Guid.Parse("00000001-710e-4a5b-8d75-3e5b444bc3cf")) )
                        {
                            var characteristics = await service.GetCharacteristicsAsync();

                            foreach (var characteristic in characteristics)
                            {
                                Debug.WriteLine($"{characteristic.Uuid} | {characteristic.Name}");
                                if (characteristic.Uuid.Equals(rxUUID))
                                {
                                    var result = await ReadOperationAsync(characteristic);

                                }
                                else if (characteristic.Uuid.Equals(wxUUID))
                                {
                                    const string data = "C";
                                    await WriteOperationAsync(characteristic,data);

                                }
                            }
                        }
                    }
                    var transmissionService = services.FirstOrDefault(s => s.Id == Guid.Parse(SendReceiveServiceUUID));
                    var readCharacteristic = await transmissionService.GetCharacteristicAsync(Guid.Parse(rxUUID));

                    var result2 = await ReadOperationAsync(readCharacteristic);

                }
                else
                {
                    Debug.WriteLine("Failed to connect to the device.");
                    IsConnected = false;
                }
            }
            catch (DeviceConnectionException ex)
            {
                Debug.WriteLine($"Connection error: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"Failed to connect: {ex.Message}", "OK");
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"General connection error: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"An error occurred: {ex.Message}", "OK");
            }
            finally
            {
                // Ensure we reset connection state
                if (_connectedDevice == null)
                {
                    IsConnected = false;
                    // Consider resetting any other related UI state
                }
            }
        }

       


        private async Task<string> ReadOperationAsync(ICharacteristic characteristic, int retryCount = 3)
        {
            for (int attempt = 1; attempt <= retryCount; attempt++)
            {
                try
                {
                    Debug.WriteLine($"Attempt {attempt}: Reading from characteristic {characteristic.Uuid}");
                    var temperatureData = await characteristic.ReadAsync();
                    var temperatureValue = System.Text.Encoding.UTF8.GetString(temperatureData.data);
                    Debug.WriteLine($"Temperature Value: {temperatureValue}");
                    return temperatureValue;
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"Read attempt {attempt} failed: {ex.Message}");
                    if (attempt == retryCount)
                    {
                        Debug.WriteLine("Max retry attempts reached for reading.");
                        throw; 
                    } else
                    {
                        await Task.Delay(1000);
                    }
                }
            }
            return null; 
        }

        private async Task WriteOperationAsync(ICharacteristic characteristic,string sendData, int retryCount = 3)
        {
            byte[] data = System.Text.Encoding.UTF8.GetBytes(sendData);

            for (int attempt = 1; attempt <= retryCount; attempt++)
            {
                try
                {
                    Debug.WriteLine($"Attempt {attempt}: Writing to characteristic {characteristic.Uuid}");
                    await characteristic.WriteAsync(data);
                    Debug.WriteLine("Write operation succeeded.");
                    return;
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"Write attempt {attempt} failed: {ex.Message}");
                    if (attempt == retryCount)
                    {
                        Debug.WriteLine("Max retry attempts reached for writing.");
                        throw; 
                    } else
                    {
                        await Task.Delay(1000);
                    }
                }
            }
        }


        private async void OnDisconnect()
        {
            if (_connectedDevice != null)
            {
                try
                {
                    Debug.WriteLine($"Attempting to disconnect from {_connectedDevice.Name}");
                    await _adapter.DisconnectDeviceAsync(_connectedDevice);
                    Debug.WriteLine("Disconnected successfully.");
                    IsWifiNetworkSelectionVisible = false;
                    // Dispose of the device reference
                    if (_connectedDevice != null)
                        _connectedDevice.Dispose();
                    _connectedDevice = null;
                    IsConnected = false;
                    ScanDevices();
                    await Application.Current.MainPage.DisplayAlert("Disconnected", "Device disconnected successfully.", "OK");
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"Error during disconnection: {ex.Message}");
                    await Application.Current.MainPage.DisplayAlert("Disconnection Error", $"Failed to disconnect: {ex.Message}", "OK");
                }
            }
            else
            {
                Debug.WriteLine("No device to disconnect.");
            }
        }
   

    }


    /**
     * 
                var transmitService = services.FirstOrDefault(s => s.Id == Guid.Parse(SendReceiveServiceUUID));
                Debug.WriteLine("Transmit service was accessed");
                Debug.WriteLine($"{transmitService.Name} : {transmitService.Id}");
                if (transmitService == null)
                {
                    Debug.WriteLine("Transmit service wasn't found");
                }
                var characteristics = await transmitService.GetCharacteristicsAsync();

                foreach (var characteristic in characteristics)
                {
                    Debug.WriteLine($"{characteristic.Uuid} | {characteristic.Name}");
                    if (characteristic.Uuid.Equals(rxUUID))
                    {
                        var result = await ReadOperationAsync(characteristic);

                    }
                    else if (characteristic.Uuid.Equals(wxUUID))
                    {
                        //await WriteOperationAsync(characteristic);

                    }
                }

            }
            catch (DeviceConnectionException ex)
            {
                Debug.WriteLine($"Connection error: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"Failed to connect: {ex.Message}", "OK");
            }
            catch (TaskCanceledException ex)
            {
                Debug.WriteLine($"TaskCanceledException: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", "Connection timed out.", "OK");
                _connectedDevice = null;
                IsConnected = false;
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"General connection error: {ex.Message}");
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"An error occurred: {ex.Message}", "OK");
            }
        } **/
}
